//+------------------------------------------------------------------+
//|                                                     iMA_Dif.mq4 |
//|                                Copyright  2006, Robert Hill |
//+------------------------------------------------------------------+
// This displays the difference between two MAs,
// Added MAType 4 to use LSMA
#property copyright "Copyright  2006, Robert Hill"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Blue


//---- input parameters
extern int       FastMAType=0; //0=SMA, 1=EMA, 2=SMMA, 3=LWMA, 4=LSMA
extern int       FastMAPeriod=5;
extern int       FastMAAppliedPrice=4;//0=close, 1=open, 2=high, 3=low, 4=median(high+low)/2, 5=typical(high+low+close)/3, 6=weighted(high+low+close+close)/4
extern int       SlowMAPeriod=21;
extern int       SlowMAType=1; //0=SMA, 1=EMA, 2=SMMA, 3=LWMA, 4=LSMA
extern int       SlowMAAppliedPrice=4;//0=close, 1=open, 2=high, 3=low, 4=median(high+low)/2, 5=typical(high+low+close)/3, 6=weighted(high+low+close+close)/4

//---- buffers
double MA_DifBuffer[];
int    FastMAMode;
int    SlowMAMode;
string strFastMAType;
string strSlowMAType;

//---- variables



//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
switch (FastMAType)
   {
      case 1: strFastMAType="EMA"; FastMAMode=MODE_EMA; break;
      case 2: strFastMAType="SMMA"; FastMAMode=MODE_SMMA; break;
      case 3: strFastMAType="LWMA"; FastMAMode=MODE_LWMA; break;
      case 4: strFastMAType="LSMA"; break;
      default: strFastMAType="SMA"; FastMAMode=MODE_SMA; break;
   }

switch (SlowMAType)
   {
      case 1: strSlowMAType="EMA"; SlowMAMode=MODE_EMA; break;
      case 2: strSlowMAType="SMMA"; SlowMAMode=MODE_SMMA; break;
      case 3: strSlowMAType="LWMA"; SlowMAMode=MODE_LWMA; break;
      case 4: strSlowMAType="LSMA"; break;
      default: strSlowMAType="SMA"; SlowMAMode=MODE_SMA; break;
   }
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
   //---- indicators
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,MA_DifBuffer);
   SetIndexDrawBegin(0,SlowMAPeriod);
   //---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MA Diff ("+FastMAPeriod+"-"+strFastMAType+","+SlowMAPeriod+"-"+strSlowMAType+") ");
   SetIndexLabel(0,"DPO");
   SetLevelStyle(STYLE_DOT,1,Silver);
   SetLevelValue(0,0);
   
   //----
   return(0);
}
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   //---- 
   
   //----
   return(0);
}

//+------------------------------------------------------------------+
//| LSMA with PriceMode                                              |
//| PrMode  0=close, 1=open, 2=high, 3=low, 4=median(high+low)/2,    |
//| 5=typical(high+low+close)/3, 6=weighted(high+low+close+close)/4  |
//+------------------------------------------------------------------+

double LSMA(int Rperiod, int prMode, int shift)
{
   int i;
   double sum, pr;
   int length;
   double lengthvar;
   double tmp;
   double wt;

   length = Rperiod;
 
   sum = 0;
   for(i = length; i >= 1  ; i--)
   {
     lengthvar = length + 1;
     lengthvar /= 3;
     tmp = 0;
     switch (prMode)
     {
     case 0: pr = Close[length-i+shift];break;
     case 1: pr = Open[length-i+shift];break;
     case 2: pr = High[length-i+shift];break;
     case 3: pr = Low[length-i+shift];break;
     case 4: pr = (High[length-i+shift] + Low[length-i+shift])/2;break;
     case 5: pr = (High[length-i+shift] + Low[length-i+shift] + Close[length-i+shift])/3;break;
     case 6: pr = (High[length-i+shift] + Low[length-i+shift] + Close[length-i+shift] + Close[length-i+shift])/4;break;
     }
     tmp = ( i - lengthvar)*pr;
     sum+=tmp;
    }
    wt = sum*6/(length*(length+1));
    
    return(wt);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   double Fast, Slow;
   int limit;
   int counted_bars = IndicatorCounted();
   //---- check for possible errors
   if (counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   if (counted_bars>0) counted_bars--;
   limit = Bars - counted_bars;

   for(int i=limit; i>=0; i--)
   {
      if (FastMAType == 4)
      {
        Fast = LSMA(FastMAPeriod,FastMAAppliedPrice,i);
      }
      else
      {
        Fast = iMA(NULL,0,FastMAPeriod,0,FastMAMode,FastMAAppliedPrice,i);
      }
      if (SlowMAType == 4)
      {
        Slow = LSMA(SlowMAPeriod,SlowMAAppliedPrice,i);
      }
      else
      {
        Slow = iMA(NULL,0,SlowMAPeriod,0,SlowMAMode,SlowMAAppliedPrice,i);
      }
      
      MA_DifBuffer[i] = MathRound((Fast - Slow)/Point);

   }
   
   //----
   return(0);
}
//+------------------------------------------------------------------+